# Functional PHP

## Running Tests

This project uses [Composer](getcomposer.org) for dependency management and
[PHPUnit](phpunit.de) for unit testing.  PHPUnit requires access to autoloading
and the autoloading files must be first be generated by composer with the
following command:

    $ composer dump-autoload

Then, run the unit tests with

    $ phpunit --bootstrap vendor/autoload.php test

## Usage

Currently this library only supports Functors and Monads and, in particular,
only supports the Maybe version of these types.  If you're familiar with Maybe
Functors and Monads from another language such as Haskell, then this should be
intuitive.

Maybe is intended to be used in lieu of both exception and null handling.  Using
Maybe allows one to completely avoid having to wrap code in try/catch or check
for null values.

The Maybe type in this library deviates slightly from that found in Haskell.
It's really more of a combination of Either and Maybe in that in this library
Nothing takes an optional parameter (which is typically an error message).  So
Maybe is used for error handling but you can also have an error message
associated with errors.

### Functor

Functors are used for mapping regular functions over values in a context.  In
this case the context is Maybe.  This library implements Functor with the
`Functor` interface which contains only one method, namely `fmap`.  This is best illustrated by an example.  Say we have a string in a Maybe context:

```php
use TMciver\Functional\Just;

$maybeAString = new Just("Hello, world!");
```

and we want to run a string-manipulation function over it:

```php
$maybeAnotherString = $maybeAString->fmap(function ($s) {
   return strtoupper($s);
});
```

then, `$maybeAnotherString` will be `Just("HELLO, WORLD!")`.

### Monad

Monad is similar to Functor.  It is implemented using the `Monad` interface
which also contains only a single method, namely `bind`.  But whereas `fmap`
takes a _normal_ function, `bind` takes a function that returns a `Monad` of the
same type on which `bind` is currently being called.

For example, say we have an integer contained in a `Just`:

```php
use TMciver\Functional\Just;

$maybeAnInt = new Just(1);
```

and we want to apply the following function to the contained integer:

```php
function maybeAddOne($i) {
	return new Just($i + 1);
}
```

(note that this function expects an integer argument - not an integer wrapped in
a Maybe).  Then you simply do

```php
$maybeAnIntPlusOne = $maybeAnInt->bind(function($j) {
	return maybeAddOne($j);
});
```

Note that `bind` can be called with any `callable` so the above could also have
been written:

```php
$maybeAnIntPlusOne = $maybeAnInt->bind('maybeAddOne');
```

See
[PHP's documentation on `callable`](http://php.net/manual/en/language.types.callable.php)
for more info.  If calling a function using a string seems strange, good; it
*is* strange! :)

### Applicative

There currently exists an `Applicative` interface but it contains no methods.
Applicative functionality will be added in the future.
